"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
SPDX-License-Identifier: Apache-2.0"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import substrait.algebra_pb2
import substrait.extensions.extensions_pb2
import substrait.plan_pb2
import substrait.type_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ExpressionReference(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXPRESSION_FIELD_NUMBER: builtins.int
    MEASURE_FIELD_NUMBER: builtins.int
    OUTPUT_NAMES_FIELD_NUMBER: builtins.int
    @property
    def expression(self) -> substrait.algebra_pb2.Expression: ...
    @property
    def measure(self) -> substrait.algebra_pb2.AggregateFunction: ...
    @property
    def output_names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """Field names in depth-first order"""

    def __init__(
        self,
        *,
        expression: substrait.algebra_pb2.Expression | None = ...,
        measure: substrait.algebra_pb2.AggregateFunction | None = ...,
        output_names: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["expr_type", b"expr_type", "expression", b"expression", "measure", b"measure"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["expr_type", b"expr_type", "expression", b"expression", "measure", b"measure", "output_names", b"output_names"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["expr_type", b"expr_type"]) -> typing.Literal["expression", "measure"] | None: ...

Global___ExpressionReference: typing_extensions.TypeAlias = ExpressionReference

@typing.final
class ExtendedExpression(google.protobuf.message.Message):
    """Describe a set of operations to complete.
    For compactness sake, identifiers are normalized at the plan level.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    EXTENSION_URIS_FIELD_NUMBER: builtins.int
    EXTENSION_URNS_FIELD_NUMBER: builtins.int
    EXTENSIONS_FIELD_NUMBER: builtins.int
    REFERRED_EXPR_FIELD_NUMBER: builtins.int
    BASE_SCHEMA_FIELD_NUMBER: builtins.int
    ADVANCED_EXTENSIONS_FIELD_NUMBER: builtins.int
    EXPECTED_TYPE_URLS_FIELD_NUMBER: builtins.int
    @property
    def version(self) -> substrait.plan_pb2.Version:
        """Substrait version of the expression. Optional up to 0.17.0, required for later
        versions.
        """

    @property
    def extension_uris(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[substrait.extensions.extensions_pb2.SimpleExtensionURI]:
        """a list of yaml specifications this expression may depend on
        this is now deprecated and extension_urns should be used instead.
        """

    @property
    def extension_urns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[substrait.extensions.extensions_pb2.SimpleExtensionURN]:
        """a list of extension specifications this expression may depend on,
        referenced by Extension URN
        """

    @property
    def extensions(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[substrait.extensions.extensions_pb2.SimpleExtensionDeclaration]:
        """a list of extensions this expression may depend on"""

    @property
    def referred_expr(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___ExpressionReference]:
        """one or more expression trees with same order in plan rel"""

    @property
    def base_schema(self) -> substrait.type_pb2.NamedStruct: ...
    @property
    def advanced_extensions(self) -> substrait.extensions.extensions_pb2.AdvancedExtension:
        """additional extensions associated with this expression."""

    @property
    def expected_type_urls(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """A list of com.google.Any entities that this plan may use. Can be used to
        warn if some embedded message types are unknown. Note that this list may
        include message types that are ignorable (optimizations) or that are
        unused. In many cases, a consumer may be able to work with a plan even if
        one or more message types defined here are unknown.
        """

    def __init__(
        self,
        *,
        version: substrait.plan_pb2.Version | None = ...,
        extension_uris: collections.abc.Iterable[substrait.extensions.extensions_pb2.SimpleExtensionURI] | None = ...,
        extension_urns: collections.abc.Iterable[substrait.extensions.extensions_pb2.SimpleExtensionURN] | None = ...,
        extensions: collections.abc.Iterable[substrait.extensions.extensions_pb2.SimpleExtensionDeclaration] | None = ...,
        referred_expr: collections.abc.Iterable[Global___ExpressionReference] | None = ...,
        base_schema: substrait.type_pb2.NamedStruct | None = ...,
        advanced_extensions: substrait.extensions.extensions_pb2.AdvancedExtension | None = ...,
        expected_type_urls: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["advanced_extensions", b"advanced_extensions", "base_schema", b"base_schema", "version", b"version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["advanced_extensions", b"advanced_extensions", "base_schema", b"base_schema", "expected_type_urls", b"expected_type_urls", "extension_uris", b"extension_uris", "extension_urns", b"extension_urns", "extensions", b"extensions", "referred_expr", b"referred_expr", "version", b"version"]) -> None: ...

Global___ExtendedExpression: typing_extensions.TypeAlias = ExtendedExpression



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: substrait/plan.proto
# Protobuf Python Version: 5.29.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    5,
    '',
    'substrait/plan.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from substrait import algebra_pb2 as substrait_dot_algebra__pb2
from substrait.extensions import extensions_pb2 as substrait_dot_extensions_dot_extensions__pb2
from substrait import type_pb2 as substrait_dot_type__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14substrait/plan.proto\x12\tsubstrait\x1a\x17substrait/algebra.proto\x1a%substrait/extensions/extensions.proto\x1a\x14substrait/type.proto\"c\n\x07PlanRel\x12\"\n\x03rel\x18\x01 \x01(\x0b\x32\x0e.substrait.RelH\x00R\x03rel\x12(\n\x04root\x18\x02 \x01(\x0b\x32\x12.substrait.RelRootH\x00R\x04rootB\n\n\x08rel_type\"\xf2\x04\n\x04Plan\x12,\n\x07version\x18\x06 \x01(\x0b\x32\x12.substrait.VersionR\x07version\x12S\n\x0e\x65xtension_uris\x18\x01 \x03(\x0b\x32(.substrait.extensions.SimpleExtensionURIB\x02\x18\x01R\rextensionUris\x12O\n\x0e\x65xtension_urns\x18\x08 \x03(\x0b\x32(.substrait.extensions.SimpleExtensionURNR\rextensionUrns\x12P\n\nextensions\x18\x02 \x03(\x0b\x32\x30.substrait.extensions.SimpleExtensionDeclarationR\nextensions\x12\x30\n\trelations\x18\x03 \x03(\x0b\x32\x12.substrait.PlanRelR\trelations\x12X\n\x13\x61\x64vanced_extensions\x18\x04 \x01(\x0b\x32\'.substrait.extensions.AdvancedExtensionR\x12\x61\x64vancedExtensions\x12,\n\x12\x65xpected_type_urls\x18\x05 \x03(\tR\x10\x65xpectedTypeUrls\x12Q\n\x12parameter_bindings\x18\x07 \x03(\x0b\x32\".substrait.DynamicParameterBindingR\x11parameterBindings\x12\x37\n\x0ctype_aliases\x18\t \x03(\x0b\x32\x14.substrait.TypeAliasR\x0btypeAliases\";\n\x0bPlanVersion\x12,\n\x07version\x18\x06 \x01(\x0b\x32\x12.substrait.VersionR\x07version\"\xa9\x01\n\x07Version\x12!\n\x0cmajor_number\x18\x01 \x01(\rR\x0bmajorNumber\x12!\n\x0cminor_number\x18\x02 \x01(\rR\x0bminorNumber\x12!\n\x0cpatch_number\x18\x03 \x01(\rR\x0bpatchNumber\x12\x19\n\x08git_hash\x18\x04 \x01(\tR\x07gitHash\x12\x1a\n\x08producer\x18\x05 \x01(\tR\x08producer\"y\n\x17\x44ynamicParameterBinding\x12)\n\x10parameter_anchor\x18\x01 \x01(\rR\x0fparameterAnchor\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32\x1d.substrait.Expression.LiteralR\x05valueBf\n\x12io.substrait.protoP\x01Z9github.com/substrait-io/substrait-protobuf/go/substraitpb\xaa\x02\x12Substrait.Protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'substrait.plan_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022io.substrait.protoP\001Z9github.com/substrait-io/substrait-protobuf/go/substraitpb\252\002\022Substrait.Protobuf'
  _globals['_PLAN'].fields_by_name['extension_uris']._loaded_options = None
  _globals['_PLAN'].fields_by_name['extension_uris']._serialized_options = b'\030\001'
  _globals['_PLANREL']._serialized_start=121
  _globals['_PLANREL']._serialized_end=220
  _globals['_PLAN']._serialized_start=223
  _globals['_PLAN']._serialized_end=849
  _globals['_PLANVERSION']._serialized_start=851
  _globals['_PLANVERSION']._serialized_end=910
  _globals['_VERSION']._serialized_start=913
  _globals['_VERSION']._serialized_end=1082
  _globals['_DYNAMICPARAMETERBINDING']._serialized_start=1084
  _globals['_DYNAMICPARAMETERBINDING']._serialized_end=1205
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
SPDX-License-Identifier: Apache-2.0"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.empty_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Type(google.protobuf.message.Message):
    """Note: type_variation_reference fields within Type messages reference a
    type_variation_anchor defined in the plan's extension declarations.

    The value 0 represents the system-preferred variation and is a valid reference value.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Nullability:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _NullabilityEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Type._Nullability.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        NULLABILITY_UNSPECIFIED: Type._Nullability.ValueType  # 0
        NULLABILITY_NULLABLE: Type._Nullability.ValueType  # 1
        NULLABILITY_REQUIRED: Type._Nullability.ValueType  # 2

    class Nullability(_Nullability, metaclass=_NullabilityEnumTypeWrapper): ...
    NULLABILITY_UNSPECIFIED: Type.Nullability.ValueType  # 0
    NULLABILITY_NULLABLE: Type.Nullability.ValueType  # 1
    NULLABILITY_REQUIRED: Type.Nullability.ValueType  # 2

    @typing.final
    class Boolean(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class I8(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class I16(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class I32(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class I64(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class FP32(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class FP64(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class String(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Binary(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Timestamp(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Date(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Time(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class TimestampTZ(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class IntervalYear(google.protobuf.message.Message):
        """An interval consisting of years and months"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class IntervalDay(google.protobuf.message.Message):
        """An interval consisting of days, seconds, and microseconds"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        PRECISION_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        precision: builtins.int
        """Sub-second precision, 0 means the value given is in seconds, 3 is milliseconds, 6 microseconds, 9 is nanoseconds, etc.
        if unset, treat as 6.
        """
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
            precision: builtins.int | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["_precision", b"_precision", "precision", b"precision"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["_precision", b"_precision", "nullability", b"nullability", "precision", b"precision", "type_variation_reference", b"type_variation_reference"]) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["_precision", b"_precision"]) -> typing.Literal["precision"] | None: ...

    @typing.final
    class IntervalCompound(google.protobuf.message.Message):
        """An interval consisting of the components of both IntervalYear and IntervalDay"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        PRECISION_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        precision: builtins.int
        """Sub-second precision, 0 means the value given is in seconds, 3 is milliseconds, 6 microseconds, 9 is nanoseconds, etc."""
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
            precision: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "precision", b"precision", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class UUID(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class FixedChar(google.protobuf.message.Message):
        """Start compound types."""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LENGTH_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        length: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            length: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["length", b"length", "nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class VarChar(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LENGTH_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        length: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            length: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["length", b"length", "nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class FixedBinary(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LENGTH_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        length: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            length: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["length", b"length", "nullability", b"nullability", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Decimal(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        SCALE_FIELD_NUMBER: builtins.int
        PRECISION_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        scale: builtins.int
        precision: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            scale: builtins.int = ...,
            precision: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "precision", b"precision", "scale", b"scale", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class PrecisionTime(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PRECISION_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        precision: builtins.int
        """Sub-second precision, 0 means the value given is in seconds, 3 is milliseconds, 6 microseconds, 9 is nanoseconds, 12 is picoseconds"""
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            precision: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "precision", b"precision", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class PrecisionTimestamp(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PRECISION_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        precision: builtins.int
        """Sub-second precision, 0 means the value given is in seconds, 3 is milliseconds, 6 microseconds, 9 is nanoseconds, 12 is picoseconds"""
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            precision: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "precision", b"precision", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class PrecisionTimestampTZ(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PRECISION_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        precision: builtins.int
        """Sub-second precision, 0 means the value given is in seconds, 3 is milliseconds, 6 microseconds, 9 is nanoseconds, 12 is picoseconds"""
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        def __init__(
            self,
            *,
            precision: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "precision", b"precision", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Struct(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPES_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        @property
        def types(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___Type]: ...
        def __init__(
            self,
            *,
            types: collections.abc.Iterable[Global___Type] | None = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_variation_reference", b"type_variation_reference", "types", b"types"]) -> None: ...

    @typing.final
    class List(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        @property
        def type(self) -> Global___Type: ...
        def __init__(
            self,
            *,
            type: Global___Type | None = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["type", b"type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type", b"type", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Map(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        @property
        def key(self) -> Global___Type: ...
        @property
        def value(self) -> Global___Type: ...
        def __init__(
            self,
            *,
            key: Global___Type | None = ...,
            value: Global___Type | None = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "nullability", b"nullability", "type_variation_reference", b"type_variation_reference", "value", b"value"]) -> None: ...

    @typing.final
    class Func(google.protobuf.message.Message):
        """A function type for higher-order functions.
        Represents a function that takes parameters of specified types and
        returns a value of a specified type.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PARAMETER_TYPES_FIELD_NUMBER: builtins.int
        RETURN_TYPE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        nullability: Global___Type.Nullability.ValueType
        """Nullability of the function value itself, not its return type.
        A nullable function type (e.g., func?<i32 -> i32>) means the function
        reference may be null. This is distinct from the return type's
        nullability (e.g., func<i32 -> i32?> where the function always exists
        but may return null).
        """
        @property
        def parameter_types(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___Type]:
            """Types of parameters this function type accepts."""

        @property
        def return_type(self) -> Global___Type:
            """Return type of the function."""

        def __init__(
            self,
            *,
            parameter_types: collections.abc.Iterable[Global___Type] | None = ...,
            return_type: Global___Type | None = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["return_type", b"return_type"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "parameter_types", b"parameter_types", "return_type", b"return_type"]) -> None: ...

    @typing.final
    class UserDefined(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_REFERENCE_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        TYPE_PARAMETERS_FIELD_NUMBER: builtins.int
        type_reference: builtins.int
        """References a type_anchor defined in the plan's extension declarations."""
        type_variation_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        @property
        def type_parameters(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[Global___Type.Parameter]: ...
        def __init__(
            self,
            *,
            type_reference: builtins.int = ...,
            type_variation_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
            type_parameters: collections.abc.Iterable[Global___Type.Parameter] | None = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_parameters", b"type_parameters", "type_reference", b"type_reference", "type_variation_reference", b"type_variation_reference"]) -> None: ...

    @typing.final
    class Parameter(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        NULL_FIELD_NUMBER: builtins.int
        DATA_TYPE_FIELD_NUMBER: builtins.int
        BOOLEAN_FIELD_NUMBER: builtins.int
        INTEGER_FIELD_NUMBER: builtins.int
        ENUM_FIELD_NUMBER: builtins.int
        STRING_FIELD_NUMBER: builtins.int
        boolean: builtins.bool
        """Value parameters, like the 10 in VARCHAR<10>."""
        integer: builtins.int
        enum: builtins.str
        string: builtins.str
        @property
        def null(self) -> google.protobuf.empty_pb2.Empty:
            """Explicitly null/unspecified parameter, to select the default value (if
            any).
            """

        @property
        def data_type(self) -> Global___Type:
            """Data type parameters, like the i32 in LIST<i32>."""

        def __init__(
            self,
            *,
            null: google.protobuf.empty_pb2.Empty | None = ...,
            data_type: Global___Type | None = ...,
            boolean: builtins.bool = ...,
            integer: builtins.int = ...,
            enum: builtins.str = ...,
            string: builtins.str = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["boolean", b"boolean", "data_type", b"data_type", "enum", b"enum", "integer", b"integer", "null", b"null", "parameter", b"parameter", "string", b"string"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["boolean", b"boolean", "data_type", b"data_type", "enum", b"enum", "integer", b"integer", "null", b"null", "parameter", b"parameter", "string", b"string"]) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["parameter", b"parameter"]) -> typing.Literal["null", "data_type", "boolean", "integer", "enum", "string"] | None: ...

    @typing.final
    class TypeAliasReference(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        TYPE_ALIAS_REFERENCE_FIELD_NUMBER: builtins.int
        NULLABILITY_FIELD_NUMBER: builtins.int
        type_alias_reference: builtins.int
        nullability: Global___Type.Nullability.ValueType
        """Nullability of the referenced type alias. Must be specified."""
        def __init__(
            self,
            *,
            type_alias_reference: builtins.int = ...,
            nullability: Global___Type.Nullability.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["nullability", b"nullability", "type_alias_reference", b"type_alias_reference"]) -> None: ...

    BOOL_FIELD_NUMBER: builtins.int
    I8_FIELD_NUMBER: builtins.int
    I16_FIELD_NUMBER: builtins.int
    I32_FIELD_NUMBER: builtins.int
    I64_FIELD_NUMBER: builtins.int
    FP32_FIELD_NUMBER: builtins.int
    FP64_FIELD_NUMBER: builtins.int
    STRING_FIELD_NUMBER: builtins.int
    BINARY_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_NUMBER: builtins.int
    DATE_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    INTERVAL_YEAR_FIELD_NUMBER: builtins.int
    INTERVAL_DAY_FIELD_NUMBER: builtins.int
    INTERVAL_COMPOUND_FIELD_NUMBER: builtins.int
    TIMESTAMP_TZ_FIELD_NUMBER: builtins.int
    UUID_FIELD_NUMBER: builtins.int
    FIXED_CHAR_FIELD_NUMBER: builtins.int
    VARCHAR_FIELD_NUMBER: builtins.int
    FIXED_BINARY_FIELD_NUMBER: builtins.int
    DECIMAL_FIELD_NUMBER: builtins.int
    PRECISION_TIME_FIELD_NUMBER: builtins.int
    PRECISION_TIMESTAMP_FIELD_NUMBER: builtins.int
    PRECISION_TIMESTAMP_TZ_FIELD_NUMBER: builtins.int
    STRUCT_FIELD_NUMBER: builtins.int
    LIST_FIELD_NUMBER: builtins.int
    MAP_FIELD_NUMBER: builtins.int
    FUNC_FIELD_NUMBER: builtins.int
    USER_DEFINED_FIELD_NUMBER: builtins.int
    USER_DEFINED_TYPE_REFERENCE_FIELD_NUMBER: builtins.int
    ALIAS_FIELD_NUMBER: builtins.int
    user_defined_type_reference: builtins.int
    """Deprecated in favor of user_defined, which allows nullability and
    variations to be specified. If user_defined_type_reference is
    encountered, treat it as being non-nullable and having the default
    variation.
    """
    @property
    def bool(self) -> Global___Type.Boolean: ...
    @property
    def i8(self) -> Global___Type.I8: ...
    @property
    def i16(self) -> Global___Type.I16: ...
    @property
    def i32(self) -> Global___Type.I32: ...
    @property
    def i64(self) -> Global___Type.I64: ...
    @property
    def fp32(self) -> Global___Type.FP32: ...
    @property
    def fp64(self) -> Global___Type.FP64: ...
    @property
    def string(self) -> Global___Type.String: ...
    @property
    def binary(self) -> Global___Type.Binary: ...
    @property
    def timestamp(self) -> Global___Type.Timestamp:
        """Deprecated in favor of `PrecisionTimestamp precision_timestamp`"""

    @property
    def date(self) -> Global___Type.Date: ...
    @property
    def time(self) -> Global___Type.Time:
        """Deprecated in favor of `PrecisionTime precision_time`"""

    @property
    def interval_year(self) -> Global___Type.IntervalYear: ...
    @property
    def interval_day(self) -> Global___Type.IntervalDay: ...
    @property
    def interval_compound(self) -> Global___Type.IntervalCompound: ...
    @property
    def timestamp_tz(self) -> Global___Type.TimestampTZ:
        """Deprecated in favor of `PrecisionTimestampTZ precision_timestamp_tz`"""

    @property
    def uuid(self) -> Global___Type.UUID: ...
    @property
    def fixed_char(self) -> Global___Type.FixedChar: ...
    @property
    def varchar(self) -> Global___Type.VarChar: ...
    @property
    def fixed_binary(self) -> Global___Type.FixedBinary: ...
    @property
    def decimal(self) -> Global___Type.Decimal: ...
    @property
    def precision_time(self) -> Global___Type.PrecisionTime: ...
    @property
    def precision_timestamp(self) -> Global___Type.PrecisionTimestamp: ...
    @property
    def precision_timestamp_tz(self) -> Global___Type.PrecisionTimestampTZ:
        """value is since UNIX epoch in UTC"""

    @property
    def struct(self) -> Global___Type.Struct: ...
    @property
    def list(self) -> Global___Type.List: ...
    @property
    def map(self) -> Global___Type.Map: ...
    @property
    def func(self) -> Global___Type.Func: ...
    @property
    def user_defined(self) -> Global___Type.UserDefined: ...
    @property
    def alias(self) -> Global___Type.TypeAliasReference:
        """Reference an aliased type in `Plan.type_aliases`."""

    def __init__(
        self,
        *,
        bool: Global___Type.Boolean | None = ...,
        i8: Global___Type.I8 | None = ...,
        i16: Global___Type.I16 | None = ...,
        i32: Global___Type.I32 | None = ...,
        i64: Global___Type.I64 | None = ...,
        fp32: Global___Type.FP32 | None = ...,
        fp64: Global___Type.FP64 | None = ...,
        string: Global___Type.String | None = ...,
        binary: Global___Type.Binary | None = ...,
        timestamp: Global___Type.Timestamp | None = ...,
        date: Global___Type.Date | None = ...,
        time: Global___Type.Time | None = ...,
        interval_year: Global___Type.IntervalYear | None = ...,
        interval_day: Global___Type.IntervalDay | None = ...,
        interval_compound: Global___Type.IntervalCompound | None = ...,
        timestamp_tz: Global___Type.TimestampTZ | None = ...,
        uuid: Global___Type.UUID | None = ...,
        fixed_char: Global___Type.FixedChar | None = ...,
        varchar: Global___Type.VarChar | None = ...,
        fixed_binary: Global___Type.FixedBinary | None = ...,
        decimal: Global___Type.Decimal | None = ...,
        precision_time: Global___Type.PrecisionTime | None = ...,
        precision_timestamp: Global___Type.PrecisionTimestamp | None = ...,
        precision_timestamp_tz: Global___Type.PrecisionTimestampTZ | None = ...,
        struct: Global___Type.Struct | None = ...,
        list: Global___Type.List | None = ...,
        map: Global___Type.Map | None = ...,
        func: Global___Type.Func | None = ...,
        user_defined: Global___Type.UserDefined | None = ...,
        user_defined_type_reference: builtins.int = ...,
        alias: Global___Type.TypeAliasReference | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["alias", b"alias", "binary", b"binary", "bool", b"bool", "date", b"date", "decimal", b"decimal", "fixed_binary", b"fixed_binary", "fixed_char", b"fixed_char", "fp32", b"fp32", "fp64", b"fp64", "func", b"func", "i16", b"i16", "i32", b"i32", "i64", b"i64", "i8", b"i8", "interval_compound", b"interval_compound", "interval_day", b"interval_day", "interval_year", b"interval_year", "kind", b"kind", "list", b"list", "map", b"map", "precision_time", b"precision_time", "precision_timestamp", b"precision_timestamp", "precision_timestamp_tz", b"precision_timestamp_tz", "string", b"string", "struct", b"struct", "time", b"time", "timestamp", b"timestamp", "timestamp_tz", b"timestamp_tz", "user_defined", b"user_defined", "user_defined_type_reference", b"user_defined_type_reference", "uuid", b"uuid", "varchar", b"varchar"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["alias", b"alias", "binary", b"binary", "bool", b"bool", "date", b"date", "decimal", b"decimal", "fixed_binary", b"fixed_binary", "fixed_char", b"fixed_char", "fp32", b"fp32", "fp64", b"fp64", "func", b"func", "i16", b"i16", "i32", b"i32", "i64", b"i64", "i8", b"i8", "interval_compound", b"interval_compound", "interval_day", b"interval_day", "interval_year", b"interval_year", "kind", b"kind", "list", b"list", "map", b"map", "precision_time", b"precision_time", "precision_timestamp", b"precision_timestamp", "precision_timestamp_tz", b"precision_timestamp_tz", "string", b"string", "struct", b"struct", "time", b"time", "timestamp", b"timestamp", "timestamp_tz", b"timestamp_tz", "user_defined", b"user_defined", "user_defined_type_reference", b"user_defined_type_reference", "uuid", b"uuid", "varchar", b"varchar"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["kind", b"kind"]) -> typing.Literal["bool", "i8", "i16", "i32", "i64", "fp32", "fp64", "string", "binary", "timestamp", "date", "time", "interval_year", "interval_day", "interval_compound", "timestamp_tz", "uuid", "fixed_char", "varchar", "fixed_binary", "decimal", "precision_time", "precision_timestamp", "precision_timestamp_tz", "struct", "list", "map", "func", "user_defined", "user_defined_type_reference", "alias"] | None: ...

Global___Type: typing_extensions.TypeAlias = Type

@typing.final
class TypeAlias(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TYPE_ALIAS_ANCHOR_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    type_alias_anchor: builtins.int
    """A surrogate key used in the context of a single plan to reference a
    specific type alias.
    0 is a valid anchor/reference, but prefer non-zero values for ergonomics.
    """
    @property
    def type(self) -> Global___Type:
        """A concrete type to be aliased.

        * All type parameters must be specified.
        * Cannot directly be another alias.
        * Type parameters can reference other aliased types as long as no circular dependencies are introduced.
        * Nullability of aliased type is **ignored**. Nullability must be specified when the aliased type is referenced.
        * Type variation may be specified in the aliased type.
        """

    def __init__(
        self,
        *,
        type_alias_anchor: builtins.int = ...,
        type: Global___Type | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["type", b"type", "type_alias_anchor", b"type_alias_anchor"]) -> None: ...

Global___TypeAlias: typing_extensions.TypeAlias = TypeAlias

@typing.final
class NamedStruct(google.protobuf.message.Message):
    """A message for modeling name/type pairs.

    Useful for representing relation schemas.

    Notes:

    * The names field is in depth-first order.

    For example a schema such as:

    a: int64
    b: struct<c: float32, d: string>

    would have a `names` field that looks like:

    ["a", "b", "c", "d"]

    * Only struct fields are contained in this field's elements,
    * Map keys should be traversed first, then values when producing/consuming
    * When used to represent a relation schema, the outermost struct's
      nullability should be NULLABILITY_REQUIRED.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAMES_FIELD_NUMBER: builtins.int
    STRUCT_FIELD_NUMBER: builtins.int
    @property
    def names(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """list of names in dfs order"""

    @property
    def struct(self) -> Global___Type.Struct: ...
    def __init__(
        self,
        *,
        names: collections.abc.Iterable[builtins.str] | None = ...,
        struct: Global___Type.Struct | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["struct", b"struct"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["names", b"names", "struct", b"struct"]) -> None: ...

Global___NamedStruct: typing_extensions.TypeAlias = NamedStruct



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
SPDX-License-Identifier: Apache-2.0"""

import builtins
import collections.abc
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

if sys.version_info >= (3, 13):
    from warnings import deprecated
else:
    from typing_extensions import deprecated

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@deprecated("""This message has been marked as deprecated using proto message options.""")
@typing.final
class SimpleExtensionURI(google.protobuf.message.Message):
    """This message is deprecated, use SimpleExtensionURN moving forwards"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXTENSION_URI_ANCHOR_FIELD_NUMBER: builtins.int
    URI_FIELD_NUMBER: builtins.int
    extension_uri_anchor: builtins.int
    """A surrogate key used in the context of a single plan used to reference the
    URI associated with an extension.
    0 is a valid anchor/reference, but prefer non-zero values for ergonomics.
    """
    uri: builtins.str
    """The URI where this extension YAML can be retrieved. This is the "namespace"
    of this extension.
    """
    def __init__(
        self,
        *,
        extension_uri_anchor: builtins.int = ...,
        uri: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["extension_uri_anchor", b"extension_uri_anchor", "uri", b"uri"]) -> None: ...

Global___SimpleExtensionURI: typing_extensions.TypeAlias = SimpleExtensionURI

@typing.final
class SimpleExtensionURN(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXTENSION_URN_ANCHOR_FIELD_NUMBER: builtins.int
    URN_FIELD_NUMBER: builtins.int
    extension_urn_anchor: builtins.int
    """A surrogate key used in the context of a single plan used to reference the
    URN associated with an extension.
    0 is a valid anchor/reference, but prefer non-zero values for ergonomics.
    """
    urn: builtins.str
    """The extension URN that uniquely identifies this extension. This must follow the
    format extension:<OWNER>:<ID> and serves as the "namespace" of this extension.
    """
    def __init__(
        self,
        *,
        extension_urn_anchor: builtins.int = ...,
        urn: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["extension_urn_anchor", b"extension_urn_anchor", "urn", b"urn"]) -> None: ...

Global___SimpleExtensionURN: typing_extensions.TypeAlias = SimpleExtensionURN

@typing.final
class SimpleExtensionDeclaration(google.protobuf.message.Message):
    """Describes a mapping between a specific extension entity and the uri/urn where
    that extension can be found.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ExtensionType(google.protobuf.message.Message):
        """Describes a Type"""

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        EXTENSION_URI_REFERENCE_FIELD_NUMBER: builtins.int
        EXTENSION_URN_REFERENCE_FIELD_NUMBER: builtins.int
        TYPE_ANCHOR_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        extension_uri_reference: builtins.int
        """references the extension_uri_anchor defined for a specific extension URI.
        this is now deprecated and extension_urn_reference should be used instead.
        """
        extension_urn_reference: builtins.int
        """references the extension_urn_anchor defined for a specific extension URN.
        If both extension_urn_reference and extension_uri_reference are present,
        extension_urn_reference takes precedence.
        """
        type_anchor: builtins.int
        """A surrogate key used in the context of a single plan to reference a
        specific extension type.
        0 is a valid anchor/reference, but prefer non-zero values for ergonomics.
        """
        name: builtins.str
        """the name of the type in the defined extension YAML."""
        def __init__(
            self,
            *,
            extension_uri_reference: builtins.int = ...,
            extension_urn_reference: builtins.int = ...,
            type_anchor: builtins.int = ...,
            name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["extension_uri_reference", b"extension_uri_reference", "extension_urn_reference", b"extension_urn_reference", "name", b"name", "type_anchor", b"type_anchor"]) -> None: ...

    @typing.final
    class ExtensionTypeVariation(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        EXTENSION_URI_REFERENCE_FIELD_NUMBER: builtins.int
        EXTENSION_URN_REFERENCE_FIELD_NUMBER: builtins.int
        TYPE_VARIATION_ANCHOR_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        extension_uri_reference: builtins.int
        """references the extension_uri_anchor defined for a specific extension URI.
        this is now deprecated and extension_urn_reference should be used instead.
        """
        extension_urn_reference: builtins.int
        """references the extension_urn_anchor defined for a specific extension URN.
        If both extension_urn_reference and extension_uri_reference are present,
        extension_urn_reference takes precedence.
        """
        type_variation_anchor: builtins.int
        """A surrogate key used in the context of a single plan to reference a
        specific type variation.
        Use non-zero values; 0 is reserved for the system-preferred variation.
        """
        name: builtins.str
        """the name of the type in the defined extension YAML."""
        def __init__(
            self,
            *,
            extension_uri_reference: builtins.int = ...,
            extension_urn_reference: builtins.int = ...,
            type_variation_anchor: builtins.int = ...,
            name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["extension_uri_reference", b"extension_uri_reference", "extension_urn_reference", b"extension_urn_reference", "name", b"name", "type_variation_anchor", b"type_variation_anchor"]) -> None: ...

    @typing.final
    class ExtensionFunction(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        EXTENSION_URI_REFERENCE_FIELD_NUMBER: builtins.int
        EXTENSION_URN_REFERENCE_FIELD_NUMBER: builtins.int
        FUNCTION_ANCHOR_FIELD_NUMBER: builtins.int
        NAME_FIELD_NUMBER: builtins.int
        extension_uri_reference: builtins.int
        """references the extension_uri_anchor defined for a specific extension URI.
        this is now deprecated and extension_urn_reference should be used instead.
        """
        extension_urn_reference: builtins.int
        """references the extension_urn_anchor defined for a specific extension URN.
        If both extension_urn_reference and extension_uri_reference are present,
        extension_urn_reference takes precedence.
        """
        function_anchor: builtins.int
        """A surrogate key used in the context of a single plan to reference a
        specific function.
        0 is a valid anchor/reference, but prefer non-zero values for ergonomics.
        """
        name: builtins.str
        """A function signature"""
        def __init__(
            self,
            *,
            extension_uri_reference: builtins.int = ...,
            extension_urn_reference: builtins.int = ...,
            function_anchor: builtins.int = ...,
            name: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["extension_uri_reference", b"extension_uri_reference", "extension_urn_reference", b"extension_urn_reference", "function_anchor", b"function_anchor", "name", b"name"]) -> None: ...

    EXTENSION_TYPE_FIELD_NUMBER: builtins.int
    EXTENSION_TYPE_VARIATION_FIELD_NUMBER: builtins.int
    EXTENSION_FUNCTION_FIELD_NUMBER: builtins.int
    @property
    def extension_type(self) -> Global___SimpleExtensionDeclaration.ExtensionType: ...
    @property
    def extension_type_variation(self) -> Global___SimpleExtensionDeclaration.ExtensionTypeVariation: ...
    @property
    def extension_function(self) -> Global___SimpleExtensionDeclaration.ExtensionFunction: ...
    def __init__(
        self,
        *,
        extension_type: Global___SimpleExtensionDeclaration.ExtensionType | None = ...,
        extension_type_variation: Global___SimpleExtensionDeclaration.ExtensionTypeVariation | None = ...,
        extension_function: Global___SimpleExtensionDeclaration.ExtensionFunction | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["extension_function", b"extension_function", "extension_type", b"extension_type", "extension_type_variation", b"extension_type_variation", "mapping_type", b"mapping_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["extension_function", b"extension_function", "extension_type", b"extension_type", "extension_type_variation", b"extension_type_variation", "mapping_type", b"mapping_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["mapping_type", b"mapping_type"]) -> typing.Literal["extension_type", "extension_type_variation", "extension_function"] | None: ...

Global___SimpleExtensionDeclaration: typing_extensions.TypeAlias = SimpleExtensionDeclaration

@typing.final
class AdvancedExtension(google.protobuf.message.Message):
    """A generic object that can be used to embed additional extension information
    into the serialized substrait plan.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPTIMIZATION_FIELD_NUMBER: builtins.int
    ENHANCEMENT_FIELD_NUMBER: builtins.int
    @property
    def optimization(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[google.protobuf.any_pb2.Any]:
        """An optimization is helpful information that don't influence semantics. May
        be ignored by a consumer.
        """

    @property
    def enhancement(self) -> google.protobuf.any_pb2.Any:
        """An enhancement alter semantics. Cannot be ignored by a consumer."""

    def __init__(
        self,
        *,
        optimization: collections.abc.Iterable[google.protobuf.any_pb2.Any] | None = ...,
        enhancement: google.protobuf.any_pb2.Any | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["enhancement", b"enhancement"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["enhancement", b"enhancement", "optimization", b"optimization"]) -> None: ...

Global___AdvancedExtension: typing_extensions.TypeAlias = AdvancedExtension



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: substrait/extensions/extensions.proto
# Protobuf Python Version: 5.29.5
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    5,
    '',
    'substrait/extensions/extensions.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%substrait/extensions/extensions.proto\x12\x14substrait.extensions\x1a\x19google/protobuf/any.proto\"\\\n\x12SimpleExtensionURI\x12\x30\n\x14\x65xtension_uri_anchor\x18\x01 \x01(\rR\x12\x65xtensionUriAnchor\x12\x10\n\x03uri\x18\x02 \x01(\tR\x03uri:\x02\x18\x01\"X\n\x12SimpleExtensionURN\x12\x30\n\x14\x65xtension_urn_anchor\x18\x01 \x01(\rR\x12\x65xtensionUrnAnchor\x12\x10\n\x03urn\x18\x02 \x01(\tR\x03urn\"\xe9\x07\n\x1aSimpleExtensionDeclaration\x12g\n\x0e\x65xtension_type\x18\x01 \x01(\x0b\x32>.substrait.extensions.SimpleExtensionDeclaration.ExtensionTypeH\x00R\rextensionType\x12\x83\x01\n\x18\x65xtension_type_variation\x18\x02 \x01(\x0b\x32G.substrait.extensions.SimpleExtensionDeclaration.ExtensionTypeVariationH\x00R\x16\x65xtensionTypeVariation\x12s\n\x12\x65xtension_function\x18\x03 \x01(\x0b\x32\x42.substrait.extensions.SimpleExtensionDeclaration.ExtensionFunctionH\x00R\x11\x65xtensionFunction\x1a\xb8\x01\n\rExtensionType\x12:\n\x17\x65xtension_uri_reference\x18\x01 \x01(\rB\x02\x18\x01R\x15\x65xtensionUriReference\x12\x36\n\x17\x65xtension_urn_reference\x18\x04 \x01(\rR\x15\x65xtensionUrnReference\x12\x1f\n\x0btype_anchor\x18\x02 \x01(\rR\ntypeAnchor\x12\x12\n\x04name\x18\x03 \x01(\tR\x04name\x1a\xd4\x01\n\x16\x45xtensionTypeVariation\x12:\n\x17\x65xtension_uri_reference\x18\x01 \x01(\rB\x02\x18\x01R\x15\x65xtensionUriReference\x12\x36\n\x17\x65xtension_urn_reference\x18\x04 \x01(\rR\x15\x65xtensionUrnReference\x12\x32\n\x15type_variation_anchor\x18\x02 \x01(\rR\x13typeVariationAnchor\x12\x12\n\x04name\x18\x03 \x01(\tR\x04name\x1a\xc4\x01\n\x11\x45xtensionFunction\x12:\n\x17\x65xtension_uri_reference\x18\x01 \x01(\rB\x02\x18\x01R\x15\x65xtensionUriReference\x12\x36\n\x17\x65xtension_urn_reference\x18\x04 \x01(\rR\x15\x65xtensionUrnReference\x12\'\n\x0f\x66unction_anchor\x18\x02 \x01(\rR\x0e\x66unctionAnchor\x12\x12\n\x04name\x18\x03 \x01(\tR\x04nameB\x0e\n\x0cmapping_type\"\x85\x01\n\x11\x41\x64vancedExtension\x12\x38\n\x0coptimization\x18\x01 \x03(\x0b\x32\x14.google.protobuf.AnyR\x0coptimization\x12\x36\n\x0b\x65nhancement\x18\x02 \x01(\x0b\x32\x14.google.protobuf.AnyR\x0b\x65nhancementBq\n\x12io.substrait.protoP\x01ZDgithub.com/substrait-io/substrait-protobuf/go/substraitpb/extensions\xaa\x02\x12Substrait.Protobufb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'substrait.extensions.extensions_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\022io.substrait.protoP\001ZDgithub.com/substrait-io/substrait-protobuf/go/substraitpb/extensions\252\002\022Substrait.Protobuf'
  _globals['_SIMPLEEXTENSIONURI']._loaded_options = None
  _globals['_SIMPLEEXTENSIONURI']._serialized_options = b'\030\001'
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPE'].fields_by_name['extension_uri_reference']._loaded_options = None
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPE'].fields_by_name['extension_uri_reference']._serialized_options = b'\030\001'
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPEVARIATION'].fields_by_name['extension_uri_reference']._loaded_options = None
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPEVARIATION'].fields_by_name['extension_uri_reference']._serialized_options = b'\030\001'
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONFUNCTION'].fields_by_name['extension_uri_reference']._loaded_options = None
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONFUNCTION'].fields_by_name['extension_uri_reference']._serialized_options = b'\030\001'
  _globals['_SIMPLEEXTENSIONURI']._serialized_start=90
  _globals['_SIMPLEEXTENSIONURI']._serialized_end=182
  _globals['_SIMPLEEXTENSIONURN']._serialized_start=184
  _globals['_SIMPLEEXTENSIONURN']._serialized_end=272
  _globals['_SIMPLEEXTENSIONDECLARATION']._serialized_start=275
  _globals['_SIMPLEEXTENSIONDECLARATION']._serialized_end=1276
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPE']._serialized_start=662
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPE']._serialized_end=846
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPEVARIATION']._serialized_start=849
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONTYPEVARIATION']._serialized_end=1061
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONFUNCTION']._serialized_start=1064
  _globals['_SIMPLEEXTENSIONDECLARATION_EXTENSIONFUNCTION']._serialized_end=1260
  _globals['_ADVANCEDEXTENSION']._serialized_start=1279
  _globals['_ADVANCEDEXTENSION']._serialized_end=1412
# @@protoc_insertion_point(module_scope)
